/******************************************************************************
 *            echecs_deplacer_piece.c                                         *
 *                                                                            *
 *  Copyright  2006  gerald.dumas                                             *
 *  linechec@laposte.net                                                      *
 ******************************************************************************
 *                                                                            *
 *  This program is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by      *
 *  the Free Software Foundation; either version 2 of the License, or         *
 *  any later version.                                                        *
 *                                                                            *
 *  This program is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU General Public License for more details.                              *
 *                                                                            *
 *  You should have received a copy of the GNU General Public License         *
 *  along with this program; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*
 ******************************************************************************/

/* Renvoie vrai si la piece dans table_piece[pos_table_piece] à un coup
   jouable coup. Il n'est pas tenu compte des coups jouables avec une case
   occupée par une pièce amie
  */

#include "echecs_deplacer_piece.h"

gboolean recherche_coup_dans_liste(GList *liste_coups_jouables, gint couleur, gint position_piece, gint coup)
{
	register gint x,y;
	gint pos_table_piece=0;
	GList *liste=NULL;

	for (x=1;x<33;x++)
	{
		if (table_piece[x].type!=vide)
		{
			if (table_piece[x].place==position_piece)
				pos_table_piece=x;
		}
	}
	liste=g_list_first(liste_coups_jouables);
	y=g_list_length(liste);
	if (y>0)
	{
		for (x=0;x<y;x++)
		{
			if (((struct struct_coups_jouables*)(liste->data))->pos_piece_tab==pos_table_piece
				&& ((struct struct_coups_jouables*)(liste->data))->coup==coup
				&& ((struct struct_coups_jouables*)(liste->data))->couleur==couleur
				&& ((struct struct_coups_jouables*)(liste->data))->coup_ami==FALSE)
				return TRUE;
			liste=g_list_next(liste);
		}
	}

	return FALSE;
}

/* 	Fonction pour déplacer une pièce sur une case jouable et supprimer, le cas échéant,
	la pièce prise.
 	Cette fonction renvoie un chiffre qui correspond au déplacement effectué.
	0 : déplacement
	1 : prise
	2 : échec
	3 : petit roque
	4 : grand roque
	5 : prise en passant
	6 : prise par une piece autre qu'un pion pouvant préter à confusion ( 2 cavaliers par ex pouvant prendre la même pièce )
	7 : déplacement d'une pièce autre qu'un pion pouvant préter à confusion ( 2 cavaliers par ex pouvant aller sur la même case )
	8 : promotion simple
	9 : promotion lors d'une prise
*/

gint deplacer_piece(GList *liste_coups_jouables,type_piece piece_a_jouer,gint coordonnee_echiquier)
{
	gint w=echiquier[piece_a_jouer.place].pos_table_piece;
	gint deplacement=0;
	register gint x,y,z;
	GList *liste=NULL;

	// Vérification si c'est un simple déplacement et si ce déplacement est une pièce
	// différente d'un pion et pouvant prêter à confusion.
	if (echiquier[coordonnee_echiquier].type_piece==vide)
	{
		if (w<17) // Ceux sont les blancs
		{
			for (x=1;x<16;x++)
			{
				if (x!=w) // On ne contrôle pas la pièce jouée
				{
					if (table_piece[x].type==piece_a_jouer.type && piece_a_jouer.type!=pion) // Si une autre pièce identique existe alors
					{
						liste=g_list_first(liste_coups_jouables);
						y=g_list_length(liste);
						if (y>0)
						{
							for (z=0;z<y;z++)
							{
								if (((struct struct_coups_jouables*)(liste->data))->pos_piece_tab==x)
								{
									if (((struct struct_coups_jouables*)(liste->data))->coup_ami==FALSE)
									{
										if (((struct struct_coups_jouables*)(liste->data))->coup==coordonnee_echiquier)
										{
											deplacement=7;
										}
									}
								}
								liste=g_list_next(liste);
							}
						}
					}
				}
			}
		}
		else // Alors ceux sont les noirs
		{
			for (x=17;x<33;x++)
			{
				if (x!=w) // On ne contrôle pas la pièce jouée
				{
					if (table_piece[x].type==piece_a_jouer.type && piece_a_jouer.type!=pion) // Si une autre pièce identique existe alors
					{
						liste=g_list_first(liste_coups_jouables);
						y=g_list_length(liste);
						if (y>0)
						{
							for (z=0;z<y;z++)
							{
								if (((struct struct_coups_jouables*)(liste->data))->pos_piece_tab==x)
								{
									if (((struct struct_coups_jouables*)(liste->data))->coup_ami==FALSE)
									{
										if (((struct struct_coups_jouables*)(liste->data))->coup==coordonnee_echiquier)
										{
											deplacement=7;
										}
									}
								}
								liste=g_list_next(liste);
							}
						}
					}
				}
			}
		}	
	}

	// Si c'est une prise simple alors suppression de la piece prise dans la table
	if (echiquier[coordonnee_echiquier].type_piece!=vide)
	{
		deplacement=1;
		table_piece[echiquier[coordonnee_echiquier].pos_table_piece].type=vide;
		// Vérification si la prise ne peut pas être égale à 6 ( pour affichage )
		if (w<17) // Ceux sont les blancs
		{
			for (x=1;x<16;x++)
			{
				if (x!=w) // On ne contrôle pas la pièce jouée
				{
					if (table_piece[x].type==piece_a_jouer.type && piece_a_jouer.type!=pion) // Si une autre pièce identique existe alors
					{
						liste=g_list_first(liste_coups_jouables);
						y=g_list_length(liste);
						if (y>0)
						{
							for (z=0;z<y;z++)
							{
								if (((struct struct_coups_jouables*)(liste->data))->pos_piece_tab==x)
								{
									if (((struct struct_coups_jouables*)(liste->data))->coup_ami==FALSE)
									{
										if (((struct struct_coups_jouables*)(liste->data))->coup==coordonnee_echiquier)
										{
											deplacement=6;
										}
									}
								}
								liste=g_list_next(liste);
							}
						}
					}
				}
			}
		}
		else // Alors ceux sont les noirs
		{
			for (x=17;x<33;x++)
			{
				if (x!=w) // On ne contrôle pas la pièce jouée
				{
					if (table_piece[x].type==piece_a_jouer.type && piece_a_jouer.type!=pion) // Si une autre pièce identique existe alors
					{
						liste=g_list_first(liste_coups_jouables);
						y=g_list_length(liste);
						if (y>0)
						{
							for (z=0;z<y;z++)
							{
								if (((struct struct_coups_jouables*)(liste->data))->pos_piece_tab==x)
								{
									if (((struct struct_coups_jouables*)(liste->data))->coup_ami==FALSE)
									{
										if (((struct struct_coups_jouables*)(liste->data))->coup==coordonnee_echiquier)
										{
											deplacement=6;
										}
									}
								}
								liste=g_list_next(liste);
							}
						}
					}
				}
			}
		}
	}

	// Si c'est une prise en passant alors supp. de la pièce prise dans la table
	if ((piece_a_jouer.type==pion) && (echiquier[coordonnee_echiquier].type_piece==vide))
	{
		if (piece_a_jouer.couleur==blanche)
		{
			if (piece_a_jouer.place-coordonnee_echiquier==11)
			{
				table_piece[echiquier[piece_a_jouer.place+1].pos_table_piece].type=vide;
				deplacement=5;
			}
			if (piece_a_jouer.place-coordonnee_echiquier==13)
			{
				table_piece[echiquier[piece_a_jouer.place-1].pos_table_piece].type=vide;
				deplacement=5;
			}
		}
		else
		{
			if (coordonnee_echiquier-piece_a_jouer.place==11)
			{
				table_piece[echiquier[piece_a_jouer.place-1].pos_table_piece].type=vide;
				deplacement=5;
			}
			if (coordonnee_echiquier-piece_a_jouer.place==13)
			{
				table_piece[echiquier[piece_a_jouer.place+1].pos_table_piece].type=vide;
				deplacement=5;
			}
		}
	}

	// Contrôle si le déplacement n'est pas une promotion
	if (piece_a_jouer.type==pion)
	{
		if (piece_a_jouer.couleur==blanc && echiquier[coordonnee_echiquier].y==8)
		{
			// C'est une promotion
			deplacement=8;
			if (echiquier[coordonnee_echiquier].type_piece!=vide)
				deplacement=9;
		}
		else
		if (piece_a_jouer.couleur==noire && echiquier[coordonnee_echiquier].y==1)
		{
			// C'est une promotion
			deplacement=8;
			if (echiquier[coordonnee_echiquier].type_piece!=vide)
				deplacement=9;
		}
	}

	// Effacement de la pièce à jouer dans le tableau echiquier
	echiquier[piece_a_jouer.place].type_piece=vide;
					
	// Mise à jour de la table_piece avec la nouvelle position de la piece,
	// de la variable deplacement_initial, positionnement de la piece sur
	// l'échiquier et affichage de celle-çi dans sa nouvelle position.
	table_piece[w].place=coordonnee_echiquier;
	
	// Changement d'état de la variable "deplacement_initial"
	table_piece[w].deplacement_initial=FALSE;
	if (deplacement==8 || deplacement==9)
	table_piece[w].type=dame; // Promotion automatique en dame pour l'instant
	
	// MAJ de la pièce sur l'échiquier
	echiquier[coordonnee_echiquier].type_piece=table_piece[w].type;
	echiquier[coordonnee_echiquier].pos_table_piece=w;
	echiquier[coordonnee_echiquier].couleur_piece=table_piece[w].couleur;

	// Gestion des petits roques
	if ((piece_a_jouer.type==roi) && (coordonnee_echiquier-piece_a_jouer.place==2))
	{
		if (piece_a_jouer.couleur==blanche)
		{
			piece_a_jouer=table_piece[10];
			coordonnee_echiquier=115;
		}
		else
		{
			piece_a_jouer=table_piece[26];
			coordonnee_echiquier=31;
		}
		// récursivité pour déplacer la tour
		deplacer_piece(liste_coups_jouables,piece_a_jouer,coordonnee_echiquier);
	}

	// Gestion des grands roques
	if ((piece_a_jouer.type==roi) && (coordonnee_echiquier-piece_a_jouer.place==-2))
	{
		if (piece_a_jouer.couleur==blanche)
		{
			piece_a_jouer=table_piece[9];
			coordonnee_echiquier=113;
		}
		else
		{
			piece_a_jouer=table_piece[25];
			coordonnee_echiquier=29;
		}
		// récursivité pour déplacer la tour
		deplacer_piece(liste_coups_jouables,piece_a_jouer,coordonnee_echiquier);
	}

	// détection de roque en fonction du déplacement de la tour pour
	// que la fonction renvoie le code roque approprié.
	if ((piece_a_jouer.type==tour)
		&& (coordonnee_echiquier-piece_a_jouer.place==-2)
		&& (echiquier[piece_a_jouer.place-1].type_piece==roi))
		deplacement=3; // petit roque
	
	if ((piece_a_jouer.type==tour)
		&& (coordonnee_echiquier-piece_a_jouer.place==3)
		&& (echiquier[piece_a_jouer.place+2].type_piece==roi))
		deplacement=4; // grand roque

	// Positionnement des != drapeaux pour l'accés à l'enregistrement de la partie
	// et l'initialisation d'une nouvelle partie.
	partie_en_cours=TRUE;
	partie_enregistree=FALSE;

	return deplacement;
}
